// -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
/*****************************************************************************\
 * Computer Algebra System SINGULAR    
\*****************************************************************************/
/** @file syzextra.h
 * 
 * Computation of Syzygies
 *
 * ABSTRACT: Computation of Syzygies due to Schreyer
 *
 * @author Oleksandr Motsak
 *
 **/
/*****************************************************************************/

#ifndef SYZEXTRA_H
#define SYZEXTRA_H

// include basic definitions
#include "singularxx_defs.h"

struct  spolyrec;
typedef struct spolyrec    polyrec;
typedef polyrec *          poly;

struct ip_sring;
typedef struct ip_sring *         ring;

struct sip_sideal;
typedef struct sip_sideal *       ideal;


BEGIN_NAMESPACE_SINGULARXX    BEGIN_NAMESPACE(SYZEXTRA)

poly leadmonom(const poly p, const ring r);

/// return the tail of a given polynomial or vector
/// returns NULL if input is NULL, otherwise
/// the result is a new polynomial/vector in the ring r
poly p_Tail(const poly p, const ring r);


/// return the tail of a given ideal or module
/// returns NULL if input is NULL, otherwise
/// the result is a new ideal/module in the ring r
/// NOTE: the resulting rank is autocorrected
ideal id_Tail(const ideal id, const ring r);


/// inplace sorting of the module (ideal) id wrt >_(c,ds)
void Sort_c_ds(const ideal id, const ring r);

ideal ComputeLeadingSyzygyTerms(const ideal& id, const ring r);

ideal Compute2LeadingSyzygyTerms(const ideal& id, const ring r);


poly FindReducer(poly product, poly syzterm,
                 ideal L, ideal LS,
                 const ring r);


poly TraverseTail(poly multiplier, poly tail, 
                  ideal L, ideal T, ideal LS,
                  const ring r);

poly ReduceTerm(poly multiplier, poly term4reduction, poly syztermCheck,
                ideal L, ideal T, ideal LS, const ring r);



poly SchreyerSyzygyNF(poly syz_lead, poly syz_2, ideal L, ideal T, ideal LS, const ring r);


void ComputeSyzygy(const ideal L, const ideal T, ideal& LL, ideal& TT, const ring R);


END_NAMESPACE               END_NAMESPACE_SINGULARXX

#endif 
/* #ifndef SYZEXTRA_H */

// Vi-modeline: vim: filetype=c:syntax:shiftwidth=2:tabstop=8:textwidth=0:expandtab

